#
# This file was auto-generated by generate.py
# do not edit, manual changes might get lost.
#
# This file is part of openCARP
# (see https://www.openCARP.org).
#
# The openCARP project licenses this file to you under the
# Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
#

from .general import AbstractIonicModel as _AbstractIonicModel
from collections import OrderedDict as _OrderedDict
from carputils.format import stringtable


class PassiveIonicRegion(_AbstractIonicModel):
    """
    Describes the passive ionic model configuration

    Parameters
    ----------
    IDs : list of int
        The material tags to be included in this IMP region
    name : str
        The name of the model material region
    """
    SHORTNAME = 'passive'
    MODELID = 'PASSIVE'
    PARAMETERS = {}


class AlievPanfilovIonicModel(_AbstractIonicModel):
    """
    Describes the AlievPanfilov ionic model

    Parameters
    ----------
    IDs : list of int
        The material tags to be included in this IMP region
    name : str
        The name of the model material region
    mu1 : float, optional
        Value of mu1, defaults to 0.2
    mu2 : float, optional
        Value of mu2, defaults to 0.3
    """
    SHORTNAME = 'alievpanfilov'
    MODELID = 'AlievPanfilov'
    PARAMETERS = {'mu1': 0.2,  'mu2': 0.3}


class AslanidiSleimanIonicModel(_AbstractIonicModel):
    """
    Describes the AslanidiSleiman ionic model

    Parameters
    ----------
    IDs : list of int
        The material tags to be included in this IMP region
    name : str
        The name of the model material region
    Ca_handling : float, optional
        Value of Ca_handling, defaults to 1.0
    Cae : float, optional
        Value of Cae, defaults to 1.8
    Cle : float, optional
        Value of Cle, defaults to 150.0
    Cli : float, optional
        Value of Cli, defaults to 30.0
    GCaB : float, optional
        Value of GCaB, defaults to 0.00035182
    GCaL : float, optional
        Value of GCaL, defaults to 0.27
    GCaT : float, optional
        Value of GCaT, defaults to 0.2
    GCl : float, optional
        Value of GCl, defaults to 0.3
    GClB : float, optional
        Value of GClB, defaults to 0.0009
    GK1 : float, optional
        Value of GK1, defaults to 0.5
    GKB : float, optional
        Value of GKB, defaults to 5e-05
    GKr : float, optional
        Value of GKr, defaults to 0.015583333333333
    GKs : float, optional
        Value of GKs, defaults to 0.011738183745949
    GNa : float, optional
        Value of GNa, defaults to 20.0
    GNaB : float, optional
        Value of GNaB, defaults to 2.97e-05
    GNaL : float, optional
        Value of GNaL, defaults to 0.0162
    Gto : float, optional
        Value of Gto, defaults to 0.112
    ICaPHill : float, optional
        Value of ICaPHill, defaults to 1.6
    ICaPKmf : float, optional
        Value of ICaPKmf, defaults to 0.5
    ICaPVmf : float, optional
        Value of ICaPVmf
    INaCamax : float, optional
        Value of INaCamax, defaults to 4.5
    Ke_init : float, optional
        Value of Ke_init, defaults to 5.4
    Ki : float, optional
        Value of Ki, defaults to 135.0
    kmcaact : float, optional
        Value of kmcaact, defaults to 0.000125
    kmcai : float, optional
        Value of kmcai, defaults to 0.0036
    kmcao : float, optional
        Value of kmcao, defaults to 1.3
    kmnai1 : float, optional
        Value of kmnai1, defaults to 12.3
    kmnao : float, optional
        Value of kmnao, defaults to 87.5
    ksat : float, optional
        Value of ksat, defaults to 0.27
    Mgi : float, optional
        Value of Mgi, defaults to 1.0
    Nae : float, optional
        Value of Nae, defaults to 140.0
    Nai : float, optional
        Value of Nai, defaults to 8.8
    nu : float, optional
        Value of nu, defaults to 0.35
    PNaK : float, optional
        Value of PNaK, defaults to 0.01833
    T : float, optional
        Value of T, defaults to 310.0
    """
    SHORTNAME = 'aslanidisleiman'
    MODELID = 'AslanidiSleiman'
    PARAMETERS = {'Ca_handling': 1.0,  'Cae': 1.8,  'Cle': 150.0,  'Cli': 30.0,  
                  'GCaB': 0.00035182,  'GCaL': 0.27,  'GCaT': 0.2,  'GCl': 0.3,  
                  'GClB': 0.0009,  'GK1': 0.5,  'GKB': 5e-05,  
                  'GKr': 0.015583333333333,  'GKs': 0.011738183745949,  
                  'GNa': 20.0,  'GNaB': 2.97e-05,  'GNaL': 0.0162,  
                  'Gto': 0.112,  'ICaPHill': 1.6,  'ICaPKmf': 0.5,  
                  'ICaPVmf': None,  'INaCamax': 4.5,  'Ke_init': 5.4,  
                  'Ki': 135.0,  'kmcaact': 0.000125,  'kmcai': 0.0036,  
                  'kmcao': 1.3,  'kmnai1': 12.3,  'kmnao': 87.5,  'ksat': 0.27,  
                  'Mgi': 1.0,  'Nae': 140.0,  'Nai': 8.8,  'nu': 0.35,  
                  'PNaK': 0.01833,  'T': 310.0}


class AugustinIonicModel(_AbstractIonicModel):
    """
    Describes the Augustin ionic model

    Parameters
    ----------
    IDs : list of int
        The material tags to be included in this IMP region
    name : str
        The name of the model material region
    a : float, optional
        Value of a, defaults to 0.35
    A_1 : float, optional
        Value of A_1, defaults to -29.0
    A_2 : float, optional
        Value of A_2, defaults to 116.0
    alpha_1 : float, optional
        Value of alpha_1, defaults to 0.1
    alpha_2 : float, optional
        Value of alpha_2, defaults to 0.5
    beta_0 : float, optional
        Value of beta_0, defaults to 1.65
    beta_1 : float, optional
        Value of beta_1, defaults to -1.5
    Ca_50ref : float, optional
        Value of Ca_50ref, defaults to 0.52
    Cai_init : float, optional
        Value of Cai_init, defaults to 0.08597401
    Cao : float, optional
        Value of Cao, defaults to 1.8
    cell_type : float, optional
        Value of cell_type
    Cli : float, optional
        Value of Cli, defaults to 15.0
    Clo : float, optional
        Value of Clo, defaults to 150.0
    flags : float, optional
        Value of flags
    k_TRPN : float, optional
        Value of k_TRPN, defaults to 0.14
    k_xb : float, optional
        Value of k_xb, defaults to 0.0049
    Ki_init : float, optional
        Value of Ki_init, defaults to 120.0
    Ko : float, optional
        Value of Ko, defaults to 5.4
    lengthDep : float, optional
        Value of lengthDep, defaults to 0.0
    markov_iks : float, optional
        Value of markov_iks, defaults to 0.0
    Mgi : float, optional
        Value of Mgi, defaults to 1.0
    n_TRPN : float, optional
        Value of n_TRPN, defaults to 1.54
    n_xb : float, optional
        Value of n_xb, defaults to 3.38
    Nai_init : float, optional
        Value of Nai_init, defaults to 9.06
    Nao : float, optional
        Value of Nao, defaults to 140.0
    T_ref : float, optional
        Value of T_ref, defaults to 117.1
    TRPN_50 : float, optional
        Value of TRPN_50, defaults to 0.37
    """
    SHORTNAME = 'augustin'
    MODELID = 'Augustin'
    PARAMETERS = {'a': 0.35,  'A_1': -29.0,  'A_2': 116.0,  'alpha_1': 0.1,  
                  'alpha_2': 0.5,  'beta_0': 1.65,  'beta_1': -1.5,  
                  'Ca_50ref': 0.52,  'Cai_init': 0.08597401,  'Cao': 1.8,  
                  'cell_type': None,  'Cli': 15.0,  'Clo': 150.0,  
                  'flags': None,  'k_TRPN': 0.14,  'k_xb': 0.0049,  
                  'Ki_init': 120.0,  'Ko': 5.4,  'lengthDep': 0.0,  
                  'markov_iks': 0.0,  'Mgi': 1.0,  'n_TRPN': 1.54,  
                  'n_xb': 3.38,  'Nai_init': 9.06,  'Nao': 140.0,  
                  'T_ref': 117.1,  'TRPN_50': 0.37}


class BondarenkoIonicModel(_AbstractIonicModel):
    """
    Describes the Bondarenko ionic model

    Parameters
    ----------
    IDs : list of int
        The material tags to be included in this IMP region
    name : str
        The name of the model material region
    a : float, optional
        Value of a, defaults to 0.0625
    Acap : float, optional
        Value of Acap, defaults to 0.00013866
    aKss_init : float, optional
        Value of aKss_init, defaults to 0.287585636847048
    ato_f_init : float, optional
        Value of ato_f_init, defaults to 0.0142335908879204
    ato_s_init : float, optional
        Value of ato_s_init, defaults to 0.0443263407760382
    aur_init : float, optional
        Value of aur_init, defaults to 0.00346258606821817
    b : float, optional
        Value of b, defaults to 0.4
    Beta_Na4 : float, optional
        Value of Beta_Na4
    Beta_Na5 : float, optional
        Value of Beta_Na5
    Bmax : float, optional
        Value of Bmax, defaults to 109.0
    C_K1_init : float, optional
        Value of C_K1_init, defaults to 0.0011733433957123
    C_K2_init : float, optional
        Value of C_K2_init, defaults to 0.00105586824723736
    C_Na1_init : float, optional
        Value of C_Na1_init, defaults to 0.000420472760277688
    C_Na2_init : float, optional
        Value of C_Na2_init, defaults to 0.0240114508843199
    Cai_init : float, optional
        Value of Cai_init, defaults to 0.0949915068139801
    CaJSR_init : float, optional
        Value of CaJSR_init, defaults to 171.167969039613
    CaMKt_init : float, optional
        Value of CaMKt_init, defaults to 0.729027738385079
    CaNSR_init : float, optional
        Value of CaNSR_init, defaults to 404.825013216286
    Cao : float, optional
        Value of Cao, defaults to 1400.0
    Cass_init : float, optional
        Value of Cass_init, defaults to 0.0954184301907784
    Cm : float, optional
        Value of Cm, defaults to 1.0
    const5 : float, optional
        Value of const5, defaults to 8.2
    CSQN_tot : float, optional
        Value of CSQN_tot, defaults to 50000.0
    delta_V_L : float, optional
        Value of delta_V_L, defaults to 6.4489
    E_Cl : float, optional
        Value of E_Cl, defaults to -40.0
    eta : float, optional
        Value of eta, defaults to 0.35
    F : float, optional
        Value of F, defaults to 96.5
    GbCa : float, optional
        Value of GbCa, defaults to 0.0007
    GClCa : float, optional
        Value of GClCa, defaults to 10.0
    GK1 : float, optional
        Value of GK1, defaults to 0.35
    GKr : float, optional
        Value of GKr, defaults to 0.0165
    GKs : float, optional
        Value of GKs, defaults to 0.00575
    GKss : float, optional
        Value of GKss, defaults to 0.0595
    GKto_f : float, optional
        Value of GKto_f, defaults to 0.5347
    GKto_s : float, optional
        Value of GKto_s, defaults to 0.0
    GKur : float, optional
        Value of GKur, defaults to 0.25
    GNa : float, optional
        Value of GNa, defaults to 16.0
    GNab : float, optional
        Value of GNab, defaults to 0.0026
    I1_Na_init : float, optional
        Value of I1_Na_init, defaults to 0.000517471697712382
    I2_Na_init : float, optional
        Value of I2_Na_init, defaults to 2.45406116958509e-05
    I_init : float, optional
        Value of I_init, defaults to 0.427651445872853
    I_K_init : float, optional
        Value of I_K_init, defaults to 0.00140618453684944
    IC_Na2_init : float, optional
        Value of IC_Na2_init, defaults to 0.0174528857380179
    IC_Na3_init : float, optional
        Value of IC_Na3_init, defaults to 0.402980726914811
    ICaLmax : float, optional
        Value of ICaLmax, defaults to 7.0
    IF_Na_init : float, optional
        Value of IF_Na_init, defaults to 0.000306123648969581
    IpCa_max : float, optional
        Value of IpCa_max, defaults to 0.0955
    ito_f_init : float, optional
        Value of ito_f_init, defaults to 0.996989882138174
    ito_s_init : float, optional
        Value of ito_s_init, defaults to 0.887568880831388
    iur_init : float, optional
        Value of iur_init, defaults to 0.955684946168062
    K_L : float, optional
        Value of K_L, defaults to 0.3
    K_mAllo : float, optional
        Value of K_mAllo, defaults to 0.0
    K_mCai : float, optional
        Value of K_mCai, defaults to 3.6
    K_mCao : float, optional
        Value of K_mCao, defaults to 1400.0
    k_minus_a : float, optional
        Value of k_minus_a, defaults to 0.07125
    k_minus_b : float, optional
        Value of k_minus_b, defaults to 0.965
    k_minus_c : float, optional
        Value of k_minus_c, defaults to 0.0008
    K_mNai : float, optional
        Value of K_mNai, defaults to 12000.0
    K_mNao : float, optional
        Value of K_mNao, defaults to 88000.0
    k_plus_a : float, optional
        Value of k_plus_a, defaults to 0.006075
    k_plus_b : float, optional
        Value of k_plus_b, defaults to 0.00405
    k_plus_c : float, optional
        Value of k_plus_c, defaults to 0.009
    k_sat : float, optional
        Value of k_sat, defaults to 0.27
    kb : float, optional
        Value of kb, defaults to 0.036778
    Kd : float, optional
        Value of Kd, defaults to 0.6
    kf : float, optional
        Value of kf, defaults to 0.023761
    Ki_init : float, optional
        Value of Ki_init, defaults to 115599.50642567
    Km_Cl : float, optional
        Value of Km_Cl, defaults to 10.0
    Km_CSQN : float, optional
        Value of Km_CSQN, defaults to 630.0
    Km_Ko : float, optional
        Value of Km_Ko, defaults to 1500.0
    Km_Nai : float, optional
        Value of Km_Nai, defaults to 16600.0
    Km_pCa : float, optional
        Value of Km_pCa, defaults to 0.2885
    Km_up : float, optional
        Value of Km_up, defaults to 0.412
    Ko : float, optional
        Value of Ko, defaults to 5400.0
    m : float, optional
        Value of m, defaults to 3.0
    maxINaK : float, optional
        Value of maxINaK, defaults to 2.486
    n : float, optional
        Value of n, defaults to 4.0
    Nai_init : float, optional
        Value of Nai_init, defaults to 12364.7482121793
    Nao : float, optional
        Value of Nao, defaults to 134000.0
    nKs_init : float, optional
        Value of nKs_init, defaults to 0.00336735013094628
    O_init : float, optional
        Value of O_init, defaults to 1.23713515513533e-06
    O_K_init : float, optional
        Value of O_K_init, defaults to 0.0131742086125972
    O_Na_init : float, optional
        Value of O_Na_init, defaults to 1.46826771436314e-06
    off_rate : float, optional
        Value of off_rate, defaults to 0.0002
    on_rate : float, optional
        Value of on_rate, defaults to 0.05
    P_C2_init : float, optional
        Value of P_C2_init, defaults to 0.565182571165673
    P_CaL : float, optional
        Value of P_CaL, defaults to 2.5
    P_O1_init : float, optional
        Value of P_O1_init, defaults to 0.00571393383393735
    P_O2_init : float, optional
        Value of P_O2_init, defaults to 2.09864618235341e-08
    P_ryr_const1 : float, optional
        Value of P_ryr_const1, defaults to -0.01
    P_ryr_const2 : float, optional
        Value of P_ryr_const2, defaults to -2.0
    P_RyR_init : float, optional
        Value of P_RyR_init, defaults to 0.000280539508743811
    phi_L : float, optional
        Value of phi_L, defaults to 1.798
    R : float, optional
        Value of R, defaults to 8.314
    T : float, optional
        Value of T, defaults to 308.0
    t_L : float, optional
        Value of t_L, defaults to 1.5
    tau_i_const : float, optional
        Value of tau_i_const, defaults to 643.0
    tau_L : float, optional
        Value of tau_L, defaults to 1150.0
    tau_tr : float, optional
        Value of tau_tr, defaults to 20.0
    tau_xfer : float, optional
        Value of tau_xfer, defaults to 8.0
    v1 : float, optional
        Value of v1, defaults to 4.5
    v1_caff : float, optional
        Value of v1_caff, defaults to 10.0
    v2 : float, optional
        Value of v2, defaults to 3e-05
    v2_caff : float, optional
        Value of v2_caff, defaults to 0.1
    V_init : float, optional
        Value of V_init, defaults to -78.9452115785979
    V_L : float, optional
        Value of V_L, defaults to 0.0
    V_max_NCX : float, optional
        Value of V_max_NCX, defaults to 3.939
    VJSR : float, optional
        Value of VJSR, defaults to 7.7e-08
    vmup_init : float, optional
        Value of vmup_init, defaults to 0.5059
    Vmyo : float, optional
        Value of Vmyo, defaults to 2.2e-05
    VNSR : float, optional
        Value of VNSR, defaults to 2.31e-07
    Vss : float, optional
        Value of Vss, defaults to 2.2e-08
    y_gate_init : float, optional
        Value of y_gate_init, defaults to 0.845044436980163
    y_gate_tau_const1 : float, optional
        Value of y_gate_tau_const1, defaults to 8.0
    y_gate_tau_const2 : float, optional
        Value of y_gate_tau_const2, defaults to 315.0
    """
    SHORTNAME = 'bondarenko'
    MODELID = 'Bondarenko'
    PARAMETERS = {'a': 0.0625,  'Acap': 0.00013866,  
                  'aKss_init': 0.287585636847048,  
                  'ato_f_init': 0.0142335908879204,  
                  'ato_s_init': 0.0443263407760382,  
                  'aur_init': 0.00346258606821817,  'b': 0.4,  
                  'Beta_Na4': None,  'Beta_Na5': None,  'Bmax': 109.0,  
                  'C_K1_init': 0.0011733433957123,  
                  'C_K2_init': 0.00105586824723736,  
                  'C_Na1_init': 0.000420472760277688,  
                  'C_Na2_init': 0.0240114508843199,  
                  'Cai_init': 0.0949915068139801,  
                  'CaJSR_init': 171.167969039613,  
                  'CaMKt_init': 0.729027738385079,  
                  'CaNSR_init': 404.825013216286,  'Cao': 1400.0,  
                  'Cass_init': 0.0954184301907784,  'Cm': 1.0,  'const5': 8.2,  
                  'CSQN_tot': 50000.0,  'delta_V_L': 6.4489,  'E_Cl': -40.0,  
                  'eta': 0.35,  'F': 96.5,  'GbCa': 0.0007,  'GClCa': 10.0,  
                  'GK1': 0.35,  'GKr': 0.0165,  'GKs': 0.00575,  
                  'GKss': 0.0595,  'GKto_f': 0.5347,  'GKto_s': 0.0,  
                  'GKur': 0.25,  'GNa': 16.0,  'GNab': 0.0026,  
                  'I1_Na_init': 0.000517471697712382,  
                  'I2_Na_init': 2.45406116958509e-05,  
                  'I_init': 0.427651445872853,  
                  'I_K_init': 0.00140618453684944,  
                  'IC_Na2_init': 0.0174528857380179,  
                  'IC_Na3_init': 0.402980726914811,  'ICaLmax': 7.0,  
                  'IF_Na_init': 0.000306123648969581,  'IpCa_max': 0.0955,  
                  'ito_f_init': 0.996989882138174,  
                  'ito_s_init': 0.887568880831388,  
                  'iur_init': 0.955684946168062,  'K_L': 0.3,  'K_mAllo': 0.0,  
                  'K_mCai': 3.6,  'K_mCao': 1400.0,  'k_minus_a': 0.07125,  
                  'k_minus_b': 0.965,  'k_minus_c': 0.0008,  'K_mNai': 12000.0,  
                  'K_mNao': 88000.0,  'k_plus_a': 0.006075,  
                  'k_plus_b': 0.00405,  'k_plus_c': 0.009,  'k_sat': 0.27,  
                  'kb': 0.036778,  'Kd': 0.6,  'kf': 0.023761,  
                  'Ki_init': 115599.50642567,  'Km_Cl': 10.0,  
                  'Km_CSQN': 630.0,  'Km_Ko': 1500.0,  'Km_Nai': 16600.0,  
                  'Km_pCa': 0.2885,  'Km_up': 0.412,  'Ko': 5400.0,  'm': 3.0,  
                  'maxINaK': 2.486,  'n': 4.0,  'Nai_init': 12364.7482121793,  
                  'Nao': 134000.0,  'nKs_init': 0.00336735013094628,  
                  'O_init': 1.23713515513533e-06,  
                  'O_K_init': 0.0131742086125972,  
                  'O_Na_init': 1.46826771436314e-06,  'off_rate': 0.0002,  
                  'on_rate': 0.05,  'P_C2_init': 0.565182571165673,  
                  'P_CaL': 2.5,  'P_O1_init': 0.00571393383393735,  
                  'P_O2_init': 2.09864618235341e-08,  'P_ryr_const1': -0.01,  
                  'P_ryr_const2': -2.0,  'P_RyR_init': 0.000280539508743811,  
                  'phi_L': 1.798,  'R': 8.314,  'T': 308.0,  't_L': 1.5,  
                  'tau_i_const': 643.0,  'tau_L': 1150.0,  'tau_tr': 20.0,  
                  'tau_xfer': 8.0,  'v1': 4.5,  'v1_caff': 10.0,  'v2': 3e-05,  
                  'v2_caff': 0.1,  'V_init': -78.9452115785979,  'V_L': 0.0,  
                  'V_max_NCX': 3.939,  'VJSR': 7.7e-08,  'vmup_init': 0.5059,  
                  'Vmyo': 2.2e-05,  'VNSR': 2.31e-07,  'Vss': 2.2e-08,  
                  'y_gate_init': 0.845044436980163,  'y_gate_tau_const1': 8.0,  
                  'y_gate_tau_const2': 315.0}


class CamposIonicModel(_AbstractIonicModel):
    """
    Describes the Campos ionic model

    Parameters
    ----------
    IDs : list of int
        The material tags to be included in this IMP region
    name : str
        The name of the model material region
    Cao : float, optional
        Value of Cao, defaults to 1.8
    CSR : float, optional
        Value of CSR, defaults to 1200.0
    g_Ca : float, optional
        Value of g_Ca, defaults to 0.6
    g_K1 : float, optional
        Value of g_K1, defaults to 0.3
    g_Kr : float, optional
        Value of g_Kr, defaults to 0.003
    g_Ks : float, optional
        Value of g_Ks, defaults to 0.043
    g_Na : float, optional
        Value of g_Na, defaults to 12.0
    g_RyR : float, optional
        Value of g_RyR, defaults to 2.25
    g_sp : float, optional
        Value of g_sp
    g_tof : float, optional
        Value of g_tof, defaults to 0.1
    g_tos : float, optional
        Value of g_tos, defaults to 0.04
    INaCamax : float, optional
        Value of INaCamax, defaults to 0.3
    INaKmax : float, optional
        Value of INaKmax, defaults to 1.5
    Ko : float, optional
        Value of Ko, defaults to 5.4
    Nao : float, optional
        Value of Nao, defaults to 136.0
    v_Na : float, optional
        Value of v_Na, defaults to 0.0
    v_up : float, optional
        Value of v_up, defaults to 0.3
    """
    SHORTNAME = 'campos'
    MODELID = 'Campos'
    PARAMETERS = {'Cao': 1.8,  'CSR': 1200.0,  'g_Ca': 0.6,  'g_K1': 0.3,  
                  'g_Kr': 0.003,  'g_Ks': 0.043,  'g_Na': 12.0,  'g_RyR': 2.25,  
                  'g_sp': None,  'g_tof': 0.1,  'g_tos': 0.04,  
                  'INaCamax': 0.3,  'INaKmax': 1.5,  'Ko': 5.4,  'Nao': 136.0,  
                  'v_Na': 0.0,  'v_up': 0.3}


class CourtemancheIonicModel(_AbstractIonicModel):
    """
    Describes the Courtemanche ionic model

    Parameters
    ----------
    IDs : list of int
        The material tags to be included in this IMP region
    name : str
        The name of the model material region
    ACh : float, optional
        Value of ACh, defaults to 1e-06
    Cao : float, optional
        Value of Cao, defaults to 1.8
    Cm : float, optional
        Value of Cm, defaults to 100.0
    factorGKur : float, optional
        Value of factorGKur, defaults to 1.0
    factorGrel : float, optional
        Value of factorGrel, defaults to 1.0
    factorGtr : float, optional
        Value of factorGtr, defaults to 1.0
    factorGup : float, optional
        Value of factorGup, defaults to 1.0
    factorhGate : float, optional
        Value of factorhGate, defaults to 0.0
    factormGate : float, optional
        Value of factormGate, defaults to 0.0
    factoroaGate : float, optional
        Value of factoroaGate, defaults to 0.0
    factorxrGate : float, optional
        Value of factorxrGate, defaults to 1.0
    GACh : float, optional
        Value of GACh, defaults to 0.0
    GbCa : float, optional
        Value of GbCa, defaults to 0.00113
    GbNa : float, optional
        Value of GbNa, defaults to 0.000674
    GCaL : float, optional
        Value of GCaL, defaults to 0.1238
    GK1 : float, optional
        Value of GK1, defaults to 0.09
    GKr : float, optional
        Value of GKr, defaults to 0.0294
    GKs : float, optional
        Value of GKs, defaults to 0.129
    GNa : float, optional
        Value of GNa, defaults to 7.8
    Gto : float, optional
        Value of Gto, defaults to 0.1652
    Ko : float, optional
        Value of Ko, defaults to 5.4
    maxCaup : float, optional
        Value of maxCaup, defaults to 15.0
    maxINaCa : float, optional
        Value of maxINaCa, defaults to 1600.0
    maxINaK : float, optional
        Value of maxINaK, defaults to 0.6
    maxIpCa : float, optional
        Value of maxIpCa, defaults to 0.275
    maxIup : float, optional
        Value of maxIup, defaults to 0.005
    Nao : float, optional
        Value of Nao, defaults to 140.0
    """
    SHORTNAME = 'courtemanche'
    MODELID = 'Courtemanche'
    PARAMETERS = {'ACh': 1e-06,  'Cao': 1.8,  'Cm': 100.0,  'factorGKur': 1.0,  
                  'factorGrel': 1.0,  'factorGtr': 1.0,  'factorGup': 1.0,  
                  'factorhGate': 0.0,  'factormGate': 0.0,  
                  'factoroaGate': 0.0,  'factorxrGate': 1.0,  'GACh': 0.0,  
                  'GbCa': 0.00113,  'GbNa': 0.000674,  'GCaL': 0.1238,  
                  'GK1': 0.09,  'GKr': 0.0294,  'GKs': 0.129,  'GNa': 7.8,  
                  'Gto': 0.1652,  'Ko': 5.4,  'maxCaup': 15.0,  
                  'maxINaCa': 1600.0,  'maxINaK': 0.6,  'maxIpCa': 0.275,  
                  'maxIup': 0.005,  'Nao': 140.0}


class DiFrancescoNobleIonicModel(_AbstractIonicModel):
    """
    Describes the DiFrancescoNoble ionic model

    Parameters
    ----------
    IDs : list of int
        The material tags to be included in this IMP region
    name : str
        The name of the model material region
    Cm : float, optional
        Value of Cm, defaults to 12.0
    factorIfunny : float, optional
        Value of factorIfunny, defaults to 1.0
    GCaf : float, optional
        Value of GCaf, defaults to 1.0
    GNa : float, optional
        Value of GNa, defaults to 750.0
    Gto : float, optional
        Value of Gto, defaults to 1.0
    kNaCa : float, optional
        Value of kNaCa, defaults to 0.02
    """
    SHORTNAME = 'difrancesconoble'
    MODELID = 'DiFrancescoNoble'
    PARAMETERS = {'Cm': 12.0,  'factorIfunny': 1.0,  'GCaf': 1.0,  
                  'GNa': 750.0,  'Gto': 1.0,  'kNaCa': 0.02}


class DrouhardRobergeIonicModel(_AbstractIonicModel):
    """
    Describes the DrouhardRoberge ionic model

    Parameters
    ----------
    IDs : list of int
        The material tags to be included in this IMP region
    name : str
        The name of the model material region
    APDshorten : float, optional
        Value of APDshorten, defaults to 1.0
    GNa : float, optional
        Value of GNa, defaults to 15.0
    Gsi : float, optional
        Value of Gsi, defaults to 0.09
    """
    SHORTNAME = 'drouhardroberge'
    MODELID = 'DrouhardRoberge'
    PARAMETERS = {'APDshorten': 1.0,  'GNa': 15.0,  'Gsi': 0.09}


class FabbriIonicModel(_AbstractIonicModel):
    """
    Describes the Fabbri ionic model

    Parameters
    ----------
    IDs : list of int
        The material tags to be included in this IMP region
    name : str
        The name of the model material region
    ACh : float, optional
        Value of ACh, defaults to 0.0
    ACh_on : float, optional
        Value of ACh_on, defaults to 0.0
    C : float, optional
        Value of C, defaults to 57.0
    Ca_intracellular_fluxes_tau_dif_Ca : float, optional
        Value of Ca_intracellular_fluxes_tau_dif_Ca, defaults to 5.469e-05
    Ca_intracellular_fluxes_tau_tr : float, optional
        Value of Ca_intracellular_fluxes_tau_tr, defaults to 0.04
    Cao : float, optional
        Value of Cao, defaults to 1.8
    CCh_cas : float, optional
        Value of CCh_cas, defaults to 0.0
    CM_tot : float, optional
        Value of CM_tot, defaults to 0.045
    CQ_tot : float, optional
        Value of CQ_tot, defaults to 10.0
    delta_m : float, optional
        Value of delta_m, defaults to 1e-05
    dynamic_Ki_Nai : float, optional
        Value of dynamic_Ki_Nai, defaults to 0.0
    EC50_SK : float, optional
        Value of EC50_SK, defaults to 0.7
    EC50_SR : float, optional
        Value of EC50_SR, defaults to 0.45
    F : float, optional
        Value of F, defaults to 96485.3415
    Gf_K_max : float, optional
        Value of Gf_K_max, defaults to 0.00268
    Gf_Na_max : float, optional
        Value of Gf_Na_max, defaults to 0.00159
    GKACh : float, optional
        Value of GKACh, defaults to 0.00345
    GKr_max : float, optional
        Value of GKr_max, defaults to 0.00424
    GKs_max : float, optional
        Value of GKs_max, defaults to 0.00065
    GKur_max : float, optional
        Value of GKur_max, defaults to 0.0001539
    GNa_max : float, optional
        Value of GNa_max, defaults to 0.0223
    GSK : float, optional
        Value of GSK, defaults to 0.0
    Gto_max : float, optional
        Value of Gto_max, defaults to 0.0035
    h_inf_shift : float, optional
        Value of h_inf_shift, defaults to 0.0
    h_inf_slope : float, optional
        Value of h_inf_slope, defaults to 1.0
    h_tau_gain : float, optional
        Value of h_tau_gain, defaults to 1.0
    HSR : float, optional
        Value of HSR, defaults to 2.5
    ICaL_fCa_gate_alpha_fCa : float, optional
        Value of ICaL_fCa_gate_alpha_fCa, defaults to 0.0075
    INaK_max : float, optional
        Value of INaK_max, defaults to 0.08105
    Iso_1_uM : float, optional
        Value of Iso_1_uM, defaults to 0.0
    Iso_cas : float, optional
        Value of Iso_cas, defaults to -1.0
    Iso_linear : float, optional
        Value of Iso_linear, defaults to 0.0
    K1ni : float, optional
        Value of K1ni, defaults to 395.3
    K1no : float, optional
        Value of K1no, defaults to 1628.0
    K2ni : float, optional
        Value of K2ni, defaults to 2.289
    K2no : float, optional
        Value of K2no, defaults to 561.4
    k34 : float, optional
        Value of k34
    K3ni : float, optional
        Value of K3ni, defaults to 26.44
    K3no : float, optional
        Value of K3no, defaults to 4.663
    k_dL : float, optional
        Value of k_dL, defaults to 4.337
    k_fL : float, optional
        Value of k_fL, defaults to 5.3
    K_iso_increase : float, optional
        Value of K_iso_increase, defaults to 1.0
    K_iso_shift : float, optional
        Value of K_iso_shift, defaults to 1.0
    K_iso_shift_ninf : float, optional
        Value of K_iso_shift_ninf, defaults to 1.0
    K_iso_slope_dL : float, optional
        Value of K_iso_slope_dL, defaults to 1.0
    K_j_SRCarel : float, optional
        Value of K_j_SRCarel, defaults to 1.0
    K_j_up : float, optional
        Value of K_j_up, defaults to 1.0
    K_k1 : float, optional
        Value of K_k1, defaults to 4.64625
    K_NaCa : float, optional
        Value of K_NaCa, defaults to 3.343
    K_up : float, optional
        Value of K_up, defaults to 0.000286113
    kb_CM : float, optional
        Value of kb_CM, defaults to 542.0
    kb_CQ : float, optional
        Value of kb_CQ, defaults to 445.0
    kb_TC : float, optional
        Value of kb_TC, defaults to 446.0
    kb_TMC : float, optional
        Value of kb_TMC, defaults to 7.51
    kb_TMM : float, optional
        Value of kb_TMM, defaults to 751.0
    Kci : float, optional
        Value of Kci, defaults to 0.0207
    Kcni : float, optional
        Value of Kcni, defaults to 26.44
    Kco : float, optional
        Value of Kco, defaults to 3.663
    kf_CM : float, optional
        Value of kf_CM, defaults to 1641986.0
    kf_CQ : float, optional
        Value of kf_CQ, defaults to 175.4
    kf_TC : float, optional
        Value of kf_TC, defaults to 88800.0
    kf_TMC : float, optional
        Value of kf_TMC, defaults to 227700.0
    kf_TMM : float, optional
        Value of kf_TMM, defaults to 2277.0
    kiCa : float, optional
        Value of kiCa, defaults to 500.0
    kim : float, optional
        Value of kim, defaults to 5.0
    Km_fCa : float, optional
        Value of Km_fCa, defaults to 0.000338
    Km_Kp : float, optional
        Value of Km_Kp, defaults to 1.4
    Km_Nap : float, optional
        Value of Km_Nap, defaults to 14.0
    Ko : float, optional
        Value of Ko, defaults to 5.4
    koCa : float, optional
        Value of koCa
    kom : float, optional
        Value of kom, defaults to 660.0
    ks : float, optional
        Value of ks, defaults to 148041085.1
    L_cell : float, optional
        Value of L_cell, defaults to 67.0
    L_sub : float, optional
        Value of L_sub, defaults to 0.02
    m_inf_shift : float, optional
        Value of m_inf_shift, defaults to 0.0
    m_inf_slope : float, optional
        Value of m_inf_slope, defaults to 1.0
    m_tau_shift : float, optional
        Value of m_tau_shift, defaults to 0.0
    MaxSR : float, optional
        Value of MaxSR, defaults to 15.0
    Mgi : float, optional
        Value of Mgi, defaults to 2.5
    MinSR : float, optional
        Value of MinSR, defaults to 1.0
    n_inf_shift : float, optional
        Value of n_inf_shift, defaults to 0.0
    n_shift_slope : float, optional
        Value of n_shift_slope, defaults to 1.0
    n_SK : float, optional
        Value of n_SK, defaults to 2.2
    Nao : float, optional
        Value of Nao, defaults to 140.0
    offset_fT : float, optional
        Value of offset_fT, defaults to 0.0
    P_CaL : float, optional
        Value of P_CaL, defaults to 0.4578
    P_CaT : float, optional
        Value of P_CaT, defaults to 0.04132
    P_up_basal : float, optional
        Value of P_up_basal, defaults to 5.0
    Qci : float, optional
        Value of Qci, defaults to 0.1369
    Qco : float, optional
        Value of Qco, defaults to 0.0
    Qn : float, optional
        Value of Qn, defaults to 0.4315
    R231C_on : float, optional
        Value of R231C_on, defaults to 0.0
    R_2 : float, optional
        Value of R_2, defaults to 8314.472
    R_cell : float, optional
        Value of R_cell, defaults to 3.9
    ratio_INaL_INa : float, optional
        Value of ratio_INaL_INa, defaults to 0.0
    RTONF : float, optional
        Value of RTONF
    shift_fL : float, optional
        Value of shift_fL, defaults to 0.0
    slope_up : float, optional
        Value of slope_up, defaults to 5e-05
    T : float, optional
        Value of T, defaults to 310.0
    tau_y_a_shift : float, optional
        Value of tau_y_a_shift, defaults to 0.0
    tau_y_b_shift : float, optional
        Value of tau_y_b_shift, defaults to 0.0
    TC_tot : float, optional
        Value of TC_tot, defaults to 0.031
    TMC_tot : float, optional
        Value of TMC_tot, defaults to 0.062
    V_cell : float, optional
        Value of V_cell
    V_dL : float, optional
        Value of V_dL, defaults to -16.45085327
    V_fL : float, optional
        Value of V_fL, defaults to -37.4
    V_i : float, optional
        Value of V_i
    V_i_part : float, optional
        Value of V_i_part, defaults to 0.46
    V_jsr : float, optional
        Value of V_jsr
    V_jsr_part : float, optional
        Value of V_jsr_part, defaults to 0.0012
    V_nsr : float, optional
        Value of V_nsr
    V_nsr_part : float, optional
        Value of V_nsr_part, defaults to 0.0116
    V_sub : float, optional
        Value of V_sub
    VW_IKs : float, optional
        Value of VW_IKs, defaults to 0.0
    y_shift : float, optional
        Value of y_shift, defaults to 0.0
    """
    SHORTNAME = 'fabbri'
    MODELID = 'Fabbri'
    PARAMETERS = {'ACh': 0.0,  'ACh_on': 0.0,  'C': 57.0,  
                  'Ca_intracellular_fluxes_tau_dif_Ca': 5.469e-05,  
                  'Ca_intracellular_fluxes_tau_tr': 0.04,  'Cao': 1.8,  
                  'CCh_cas': 0.0,  'CM_tot': 0.045,  'CQ_tot': 10.0,  
                  'delta_m': 1e-05,  'dynamic_Ki_Nai': 0.0,  'EC50_SK': 0.7,  
                  'EC50_SR': 0.45,  'F': 96485.3415,  'Gf_K_max': 0.00268,  
                  'Gf_Na_max': 0.00159,  'GKACh': 0.00345,  'GKr_max': 0.00424,  
                  'GKs_max': 0.00065,  'GKur_max': 0.0001539,  
                  'GNa_max': 0.0223,  'GSK': 0.0,  'Gto_max': 0.0035,  
                  'h_inf_shift': 0.0,  'h_inf_slope': 1.0,  'h_tau_gain': 1.0,  
                  'HSR': 2.5,  'ICaL_fCa_gate_alpha_fCa': 0.0075,  
                  'INaK_max': 0.08105,  'Iso_1_uM': 0.0,  'Iso_cas': -1.0,  
                  'Iso_linear': 0.0,  'K1ni': 395.3,  'K1no': 1628.0,  
                  'K2ni': 2.289,  'K2no': 561.4,  'k34': None,  'K3ni': 26.44,  
                  'K3no': 4.663,  'k_dL': 4.337,  'k_fL': 5.3,  
                  'K_iso_increase': 1.0,  'K_iso_shift': 1.0,  
                  'K_iso_shift_ninf': 1.0,  'K_iso_slope_dL': 1.0,  
                  'K_j_SRCarel': 1.0,  'K_j_up': 1.0,  'K_k1': 4.64625,  
                  'K_NaCa': 3.343,  'K_up': 0.000286113,  'kb_CM': 542.0,  
                  'kb_CQ': 445.0,  'kb_TC': 446.0,  'kb_TMC': 7.51,  
                  'kb_TMM': 751.0,  'Kci': 0.0207,  'Kcni': 26.44,  
                  'Kco': 3.663,  'kf_CM': 1641986.0,  'kf_CQ': 175.4,  
                  'kf_TC': 88800.0,  'kf_TMC': 227700.0,  'kf_TMM': 2277.0,  
                  'kiCa': 500.0,  'kim': 5.0,  'Km_fCa': 0.000338,  
                  'Km_Kp': 1.4,  'Km_Nap': 14.0,  'Ko': 5.4,  'koCa': None,  
                  'kom': 660.0,  'ks': 148041085.1,  'L_cell': 67.0,  
                  'L_sub': 0.02,  'm_inf_shift': 0.0,  'm_inf_slope': 1.0,  
                  'm_tau_shift': 0.0,  'MaxSR': 15.0,  'Mgi': 2.5,  
                  'MinSR': 1.0,  'n_inf_shift': 0.0,  'n_shift_slope': 1.0,  
                  'n_SK': 2.2,  'Nao': 140.0,  'offset_fT': 0.0,  
                  'P_CaL': 0.4578,  'P_CaT': 0.04132,  'P_up_basal': 5.0,  
                  'Qci': 0.1369,  'Qco': 0.0,  'Qn': 0.4315,  'R231C_on': 0.0,  
                  'R_2': 8314.472,  'R_cell': 3.9,  'ratio_INaL_INa': 0.0,  
                  'RTONF': None,  'shift_fL': 0.0,  'slope_up': 5e-05,  
                  'T': 310.0,  'tau_y_a_shift': 0.0,  'tau_y_b_shift': 0.0,  
                  'TC_tot': 0.031,  'TMC_tot': 0.062,  'V_cell': None,  
                  'V_dL': -16.45085327,  'V_fL': -37.4,  'V_i': None,  
                  'V_i_part': 0.46,  'V_jsr': None,  'V_jsr_part': 0.0012,  
                  'V_nsr': None,  'V_nsr_part': 0.0116,  'V_sub': None,  
                  'VW_IKs': 0.0,  'y_shift': 0.0}


class FoxIonicModel(_AbstractIonicModel):
    """
    Describes the Fox ionic model

    Parameters
    ----------
    IDs : list of int
        The material tags to be included in this IMP region
    name : str
        The name of the model material region
    GKr : float, optional
        Value of GKr, defaults to 0.0136
    GKs : float, optional
        Value of GKs, defaults to 0.0245
    GNa : float, optional
        Value of GNa, defaults to 12.8
    PCa : float, optional
        Value of PCa, defaults to 2.26e-05
    """
    SHORTNAME = 'fox'
    MODELID = 'Fox'
    PARAMETERS = {'GKr': 0.0136,  'GKs': 0.0245,  'GNa': 12.8,  
                  'PCa': 2.26e-05}


class GaurIonicModel(_AbstractIonicModel):
    """
    Describes the Gaur ionic model

    Parameters
    ----------
    IDs : list of int
        The material tags to be included in this IMP region
    name : str
        The name of the model material region
    GK1 : float, optional
        Value of GK1
    GKb : float, optional
        Value of GKb, defaults to 0.003
    GKr : float, optional
        Value of GKr
    GKs : float, optional
        Value of GKs, defaults to 0.021
    GNa : float, optional
        Value of GNa, defaults to 14.838
    GNaL : float, optional
        Value of GNaL, defaults to 0.0075
    Gncx : float, optional
        Value of Gncx, defaults to 0.0008
    Gto : float, optional
        Value of Gto
    PCa : float, optional
        Value of PCa, defaults to 0.0002
    PNab : float, optional
        Value of PNab, defaults to 3.75e-10
    SOICR : float, optional
        Value of SOICR, defaults to 10.0
    tau_gap : float, optional
        Value of tau_gap, defaults to 3.0
    vhalf_d : float, optional
        Value of vhalf_d, defaults to 3.4
    vmyo1frac : float, optional
        Value of vmyo1frac, defaults to 0.4
    """
    SHORTNAME = 'gaur'
    MODELID = 'Gaur'
    PARAMETERS = {'GK1': None,  'GKb': 0.003,  'GKr': None,  'GKs': 0.021,  
                  'GNa': 14.838,  'GNaL': 0.0075,  'Gncx': 0.0008,  
                  'Gto': None,  'PCa': 0.0002,  'PNab': 3.75e-10,  
                  'SOICR': 10.0,  'tau_gap': 3.0,  'vhalf_d': 3.4,  
                  'vmyo1frac': 0.4}


class GrandiIonicModel(_AbstractIonicModel):
    """
    Describes the Grandi ionic model

    Parameters
    ----------
    IDs : list of int
        The material tags to be included in this IMP region
    name : str
        The name of the model material region
    Cai_init : float, optional
        Value of Cai_init, defaults to 0.08597401
    Cao : float, optional
        Value of Cao, defaults to 1.8
    cell_type : float, optional
        Value of cell_type
    Cli : float, optional
        Value of Cli, defaults to 15.0
    Clo : float, optional
        Value of Clo, defaults to 150.0
    flags : float, optional
        Value of flags
    Ki_init : float, optional
        Value of Ki_init, defaults to 120.0
    Ko : float, optional
        Value of Ko, defaults to 5.4
    markov_iks : float, optional
        Value of markov_iks, defaults to 0.0
    Mgi : float, optional
        Value of Mgi, defaults to 1.0
    Nai_init : float, optional
        Value of Nai_init, defaults to 9.06
    Nao : float, optional
        Value of Nao, defaults to 140.0
    """
    SHORTNAME = 'grandi'
    MODELID = 'Grandi'
    PARAMETERS = {'Cai_init': 0.08597401,  'Cao': 1.8,  'cell_type': None,  
                  'Cli': 15.0,  'Clo': 150.0,  'flags': None,  
                  'Ki_init': 120.0,  'Ko': 5.4,  'markov_iks': 0.0,  
                  'Mgi': 1.0,  'Nai_init': 9.06,  'Nao': 140.0}


class GrandiPanditVoigtIonicModel(_AbstractIonicModel):
    """
    Describes the GrandiPanditVoigt ionic model

    Parameters
    ----------
    IDs : list of int
        The material tags to be included in this IMP region
    name : str
        The name of the model material region
    AF : float, optional
        Value of AF, defaults to 0.0
    factor_GCaL : float, optional
        Value of factor_GCaL, defaults to 1.0
    factor_GK1 : float, optional
        Value of factor_GK1, defaults to 1.0
    factor_J_SRCarel : float, optional
        Value of factor_J_SRCarel, defaults to 1.0
    factor_J_SRleak : float, optional
        Value of factor_J_SRleak, defaults to 1.0
    factor_Vmax_SRCaP : float, optional
        Value of factor_Vmax_SRCaP, defaults to 1.0
    ISO : float, optional
        Value of ISO, defaults to 0.0
    markov_IKs : float, optional
        Value of markov_IKs, defaults to 0.0
    RA : float, optional
        Value of RA, defaults to 0.0
    usefca : float, optional
        Value of usefca, defaults to 0.0
    """
    SHORTNAME = 'grandipanditvoigt'
    MODELID = 'GrandiPanditVoigt'
    PARAMETERS = {'AF': 0.0,  'factor_GCaL': 1.0,  'factor_GK1': 1.0,  
                  'factor_J_SRCarel': 1.0,  'factor_J_SRleak': 1.0,  
                  'factor_Vmax_SRCaP': 1.0,  'ISO': 0.0,  'markov_IKs': 0.0,  
                  'RA': 0.0,  'usefca': 0.0}


class HodgkinHuxleyIonicModel(_AbstractIonicModel):
    """
    Describes the HodgkinHuxley ionic model

    Parameters
    ----------
    IDs : list of int
        The material tags to be included in this IMP region
    name : str
        The name of the model material region
    GK : float, optional
        Value of GK, defaults to 36.0
    GL : float, optional
        Value of GL, defaults to 0.3
    GNa : float, optional
        Value of GNa, defaults to 120.0
    """
    SHORTNAME = 'hodgkinhuxley'
    MODELID = 'HodgkinHuxley'
    PARAMETERS = {'GK': 36.0,  'GL': 0.3,  'GNa': 120.0}


class InadaIonicModel(_AbstractIonicModel):
    """
    Describes the Inada ionic model

    Parameters
    ----------
    IDs : list of int
        The material tags to be included in this IMP region
    name : str
        The name of the model material region
    cell_type : float, optional
        Value of cell_type
    Cm : float, optional
        Value of Cm
    E_b : float, optional
        Value of E_b
    E_CaL : float, optional
        Value of E_CaL
    flags : float, optional
        Value of flags
    Gb : float, optional
        Value of Gb
    GCaL : float, optional
        Value of GCaL
    Gf : float, optional
        Value of Gf
    GK1 : float, optional
        Value of GK1
    GKr : float, optional
        Value of GKr
    GNa : float, optional
        Value of GNa
    Gst : float, optional
        Value of Gst
    Gto : float, optional
        Value of Gto
    kNaCa : float, optional
        Value of kNaCa
    p_max : float, optional
        Value of p_max
    p_rel : float, optional
        Value of p_rel
    """
    SHORTNAME = 'inada'
    MODELID = 'Inada'
    PARAMETERS = {'cell_type': None,  'Cm': None,  'E_b': None,  'E_CaL': None,  
                  'flags': None,  'Gb': None,  'GCaL': None,  'Gf': None,  
                  'GK1': None,  'GKr': None,  'GNa': None,  'Gst': None,  
                  'Gto': None,  'kNaCa': None,  'p_max': None,  'p_rel': None}


class IribeKohlIonicModel(_AbstractIonicModel):
    """
    Describes the IribeKohl ionic model

    Parameters
    ----------
    IDs : list of int
        The material tags to be included in this IMP region
    name : str
        The name of the model material region
    Ki_init : float, optional
        Value of Ki_init, defaults to 138.22
    """
    SHORTNAME = 'iribekohl'
    MODELID = 'IribeKohl'
    PARAMETERS = {'Ki_init': 138.22}


class KurataIonicModel(_AbstractIonicModel):
    """
    Describes the Kurata ionic model

    Parameters
    ----------
    IDs : list of int
        The material tags to be included in this IMP region
    name : str
        The name of the model material region
    ACh : float, optional
        Value of ACh, defaults to 0.0
    B_max : float, optional
        Value of B_max, defaults to 0.56
    Cm : float, optional
        Value of Cm, defaults to 32.0
    GCaL : float, optional
        Value of GCaL, defaults to 0.58
    Gh : float, optional
        Value of Gh, defaults to 0.357
    GhK : float, optional
        Value of GhK, defaults to 0.6167
    GhNa : float, optional
        Value of GhNa, defaults to 0.3833
    GKACh_max : float, optional
        Value of GKACh_max
    GKr : float, optional
        Value of GKr
    GKs : float, optional
        Value of GKs, defaults to 0.0259
    GNa : float, optional
        Value of GNa, defaults to 0.0
    Gst : float, optional
        Value of Gst, defaults to 0.015
    Gsus : float, optional
        Value of Gsus, defaults to 0.02
    Gto : float, optional
        Value of Gto, defaults to 0.18
    """
    SHORTNAME = 'kurata'
    MODELID = 'Kurata'
    PARAMETERS = {'ACh': 0.0,  'B_max': 0.56,  'Cm': 32.0,  'GCaL': 0.58,  
                  'Gh': 0.357,  'GhK': 0.6167,  'GhNa': 0.3833,  
                  'GKACh_max': None,  'GKr': None,  'GKs': 0.0259,  'GNa': 0.0,  
                  'Gst': 0.015,  'Gsus': 0.02,  'Gto': 0.18}


class LoeweIonicModel(_AbstractIonicModel):
    """
    Describes the Loewe ionic model

    Parameters
    ----------
    IDs : list of int
        The material tags to be included in this IMP region
    name : str
        The name of the model material region
    ACh : float, optional
        Value of ACh, defaults to 0.0
    ACh_block : float, optional
        Value of ACh_block
    ACh_on : float, optional
        Value of ACh_on, defaults to 0.0
    ACh_shift : float, optional
        Value of ACh_shift
    C : float, optional
        Value of C, defaults to 57.0
    Ca_intracellular_fluxes_b_up : float, optional
        Value of Ca_intracellular_fluxes_b_up
    Ca_intracellular_fluxes_tau_dif_Ca : float, optional
        Value of Ca_intracellular_fluxes_tau_dif_Ca, defaults to 5.469e-05
    Ca_intracellular_fluxes_tau_tr : float, optional
        Value of Ca_intracellular_fluxes_tau_tr, defaults to 0.04
    Cao : float, optional
        Value of Cao, defaults to 1.8
    CM_tot : float, optional
        Value of CM_tot, defaults to 0.045
    CQ_tot : float, optional
        Value of CQ_tot, defaults to 10.0
    delta_m : float, optional
        Value of delta_m, defaults to 1e-05
    EC50_SK : float, optional
        Value of EC50_SK, defaults to 0.7
    EC50_SR : float, optional
        Value of EC50_SR, defaults to 0.45
    F : float, optional
        Value of F, defaults to 96485.3415
    Gf_K : float, optional
        Value of Gf_K, defaults to 0.0117
    Gf_Na : float, optional
        Value of Gf_Na, defaults to 0.00696
    GKACh : float, optional
        Value of GKACh, defaults to 0.00345
    GKr : float, optional
        Value of GKr
    GKs : float, optional
        Value of GKs
    GKs_ : float, optional
        Value of GKs_, defaults to 0.000863714
    GKur : float, optional
        Value of GKur, defaults to 7.062e-05
    GNa : float, optional
        Value of GNa, defaults to 0.0223
    GNa_L : float, optional
        Value of GNa_L, defaults to 0.0
    GSK : float, optional
        Value of GSK, defaults to 0.000165
    Gto : float, optional
        Value of Gto, defaults to 0.00167348
    HSR : float, optional
        Value of HSR, defaults to 2.5
    ICaL_fCa_gate_alpha_fCa : float, optional
        Value of ICaL_fCa_gate_alpha_fCa, defaults to 0.0075
    IKACh_a_gate_alpha_a : float, optional
        Value of IKACh_a_gate_alpha_a
    INaK_max : float, optional
        Value of INaK_max, defaults to 0.137171
    Iso_1_uM : float, optional
        Value of Iso_1_uM, defaults to 0.0
    Iso_increase_1 : float, optional
        Value of Iso_increase_1
    Iso_increase_2 : float, optional
        Value of Iso_increase_2
    Iso_shift_1 : float, optional
        Value of Iso_shift_1
    Iso_shift_2 : float, optional
        Value of Iso_shift_2
    Iso_shift_dL : float, optional
        Value of Iso_shift_dL
    Iso_slope_dL : float, optional
        Value of Iso_slope_dL
    K1ni : float, optional
        Value of K1ni, defaults to 395.3
    K1no : float, optional
        Value of K1no, defaults to 1628.0
    K2ni : float, optional
        Value of K2ni, defaults to 2.289
    K2no : float, optional
        Value of K2no, defaults to 561.4
    k34 : float, optional
        Value of k34
    K3ni : float, optional
        Value of K3ni, defaults to 26.44
    K3no : float, optional
        Value of K3no, defaults to 4.663
    k_dL : float, optional
        Value of k_dL, defaults to 5.854
    k_fL : float, optional
        Value of k_fL, defaults to 5.1737
    K_NaCa : float, optional
        Value of K_NaCa, defaults to 3.343
    K_up : float, optional
        Value of K_up, defaults to 0.000286113
    kb_CM : float, optional
        Value of kb_CM, defaults to 542.0
    kb_CQ : float, optional
        Value of kb_CQ, defaults to 445.0
    kb_TC : float, optional
        Value of kb_TC, defaults to 446.0
    kb_TMC : float, optional
        Value of kb_TMC, defaults to 7.51
    kb_TMM : float, optional
        Value of kb_TMM, defaults to 751.0
    Kci : float, optional
        Value of Kci, defaults to 0.0207
    Kcni : float, optional
        Value of Kcni, defaults to 26.44
    Kco : float, optional
        Value of Kco, defaults to 3.663
    kf_CM : float, optional
        Value of kf_CM, defaults to 1641986.0
    kf_CQ : float, optional
        Value of kf_CQ, defaults to 175.4
    kf_TC : float, optional
        Value of kf_TC, defaults to 88800.0
    kf_TMC : float, optional
        Value of kf_TMC, defaults to 227700.0
    kf_TMM : float, optional
        Value of kf_TMM, defaults to 2277.0
    kiCa : float, optional
        Value of kiCa, defaults to 500.0
    kim : float, optional
        Value of kim, defaults to 5.0
    Km_fCa : float, optional
        Value of Km_fCa, defaults to 0.000338
    Km_Kp : float, optional
        Value of Km_Kp, defaults to 1.4
    Km_Nap : float, optional
        Value of Km_Nap, defaults to 14.0
    Ko : float, optional
        Value of Ko, defaults to 5.4
    koCa : float, optional
        Value of koCa, defaults to 10000.0
    kom : float, optional
        Value of kom, defaults to 660.0
    ks : float, optional
        Value of ks, defaults to 148041085.1
    L_cell : float, optional
        Value of L_cell, defaults to 67.0
    L_sub : float, optional
        Value of L_sub, defaults to 0.02
    MaxSR : float, optional
        Value of MaxSR, defaults to 15.0
    Mgi : float, optional
        Value of Mgi, defaults to 2.5
    MinSR : float, optional
        Value of MinSR, defaults to 1.0
    n_SK : float, optional
        Value of n_SK, defaults to 2.2
    Nao : float, optional
        Value of Nao, defaults to 140.0
    offset_fT : float, optional
        Value of offset_fT, defaults to 0.0
    P_CaL : float, optional
        Value of P_CaL, defaults to 0.5046812
    P_CaT : float, optional
        Value of P_CaT, defaults to 0.04132
    P_up : float, optional
        Value of P_up
    P_up_basal : float, optional
        Value of P_up_basal, defaults to 5.0
    Qci : float, optional
        Value of Qci, defaults to 0.1369
    Qco : float, optional
        Value of Qco, defaults to 0.0
    Qn : float, optional
        Value of Qn, defaults to 0.4315
    R_2 : float, optional
        Value of R_2, defaults to 8314.472
    R_cell : float, optional
        Value of R_cell, defaults to 3.9
    RTONF : float, optional
        Value of RTONF
    shift_fL : float, optional
        Value of shift_fL, defaults to 0.0
    slope_up : float, optional
        Value of slope_up, defaults to 5e-05
    T : float, optional
        Value of T, defaults to 310.0
    TC_tot : float, optional
        Value of TC_tot, defaults to 0.031
    TMC_tot : float, optional
        Value of TMC_tot, defaults to 0.062
    V_cell : float, optional
        Value of V_cell
    V_dL : float, optional
        Value of V_dL, defaults to -7.7713
    V_i : float, optional
        Value of V_i
    V_i_part : float, optional
        Value of V_i_part, defaults to 0.46
    V_jsr : float, optional
        Value of V_jsr
    V_jsr_part : float, optional
        Value of V_jsr_part, defaults to 0.0012
    V_nsr : float, optional
        Value of V_nsr
    V_nsr_part : float, optional
        Value of V_nsr_part, defaults to 0.0116
    V_sub : float, optional
        Value of V_sub
    y_shift : float, optional
        Value of y_shift, defaults to 0.0
    """
    SHORTNAME = 'loewe'
    MODELID = 'Loewe'
    PARAMETERS = {'ACh': 0.0,  'ACh_block': None,  'ACh_on': 0.0,  
                  'ACh_shift': None,  'C': 57.0,  
                  'Ca_intracellular_fluxes_b_up': None,  
                  'Ca_intracellular_fluxes_tau_dif_Ca': 5.469e-05,  
                  'Ca_intracellular_fluxes_tau_tr': 0.04,  'Cao': 1.8,  
                  'CM_tot': 0.045,  'CQ_tot': 10.0,  'delta_m': 1e-05,  
                  'EC50_SK': 0.7,  'EC50_SR': 0.45,  'F': 96485.3415,  
                  'Gf_K': 0.0117,  'Gf_Na': 0.00696,  'GKACh': 0.00345,  
                  'GKr': None,  'GKs': None,  'GKs_': 0.000863714,  
                  'GKur': 7.062e-05,  'GNa': 0.0223,  'GNa_L': 0.0,  
                  'GSK': 0.000165,  'Gto': 0.00167348,  'HSR': 2.5,  
                  'ICaL_fCa_gate_alpha_fCa': 0.0075,  
                  'IKACh_a_gate_alpha_a': None,  'INaK_max': 0.137171,  
                  'Iso_1_uM': 0.0,  'Iso_increase_1': None,  
                  'Iso_increase_2': None,  'Iso_shift_1': None,  
                  'Iso_shift_2': None,  'Iso_shift_dL': None,  
                  'Iso_slope_dL': None,  'K1ni': 395.3,  'K1no': 1628.0,  
                  'K2ni': 2.289,  'K2no': 561.4,  'k34': None,  'K3ni': 26.44,  
                  'K3no': 4.663,  'k_dL': 5.854,  'k_fL': 5.1737,  
                  'K_NaCa': 3.343,  'K_up': 0.000286113,  'kb_CM': 542.0,  
                  'kb_CQ': 445.0,  'kb_TC': 446.0,  'kb_TMC': 7.51,  
                  'kb_TMM': 751.0,  'Kci': 0.0207,  'Kcni': 26.44,  
                  'Kco': 3.663,  'kf_CM': 1641986.0,  'kf_CQ': 175.4,  
                  'kf_TC': 88800.0,  'kf_TMC': 227700.0,  'kf_TMM': 2277.0,  
                  'kiCa': 500.0,  'kim': 5.0,  'Km_fCa': 0.000338,  
                  'Km_Kp': 1.4,  'Km_Nap': 14.0,  'Ko': 5.4,  'koCa': 10000.0,  
                  'kom': 660.0,  'ks': 148041085.1,  'L_cell': 67.0,  
                  'L_sub': 0.02,  'MaxSR': 15.0,  'Mgi': 2.5,  'MinSR': 1.0,  
                  'n_SK': 2.2,  'Nao': 140.0,  'offset_fT': 0.0,  
                  'P_CaL': 0.5046812,  'P_CaT': 0.04132,  'P_up': None,  
                  'P_up_basal': 5.0,  'Qci': 0.1369,  'Qco': 0.0,  
                  'Qn': 0.4315,  'R_2': 8314.472,  'R_cell': 3.9,  
                  'RTONF': None,  'shift_fL': 0.0,  'slope_up': 5e-05,  
                  'T': 310.0,  'TC_tot': 0.031,  'TMC_tot': 0.062,  
                  'V_cell': None,  'V_dL': -7.7713,  'V_i': None,  
                  'V_i_part': 0.46,  'V_jsr': None,  'V_jsr_part': 0.0012,  
                  'V_nsr': None,  'V_nsr_part': 0.0116,  'V_sub': None,  
                  'y_shift': 0.0}


class LuoRudy91IonicModel(_AbstractIonicModel):
    """
    Describes the LuoRudy91 ionic model

    Parameters
    ----------
    IDs : list of int
        The material tags to be included in this IMP region
    name : str
        The name of the model material region
    Cai_init : float, optional
        Value of Cai_init, defaults to 0.0002
    Gb : float, optional
        Value of Gb, defaults to 0.03921
    GK1bar : float, optional
        Value of GK1bar, defaults to 0.604
    GKbar : float, optional
        Value of GKbar, defaults to 0.282
    GKp : float, optional
        Value of GKp, defaults to 0.0183
    GNa : float, optional
        Value of GNa, defaults to 23.0
    Gsi : float, optional
        Value of Gsi, defaults to 0.09
    Ki : float, optional
        Value of Ki, defaults to 145.0
    Ko : float, optional
        Value of Ko, defaults to 5.4
    Nai : float, optional
        Value of Nai, defaults to 18.0
    Nao : float, optional
        Value of Nao, defaults to 140.0
    tau_d_factor : float, optional
        Value of tau_d_factor, defaults to 1.0
    tau_f_factor : float, optional
        Value of tau_f_factor, defaults to 1.0
    """
    SHORTNAME = 'luorudy91'
    MODELID = 'LuoRudy91'
    PARAMETERS = {'Cai_init': 0.0002,  'Gb': 0.03921,  'GK1bar': 0.604,  
                  'GKbar': 0.282,  'GKp': 0.0183,  'GNa': 23.0,  'Gsi': 0.09,  
                  'Ki': 145.0,  'Ko': 5.4,  'Nai': 18.0,  'Nao': 140.0,  
                  'tau_d_factor': 1.0,  'tau_f_factor': 1.0}


class LuoRudy94IonicModel(_AbstractIonicModel):
    """
    Describes the LuoRudy94 ionic model

    Parameters
    ----------
    IDs : list of int
        The material tags to be included in this IMP region
    name : str
        The name of the model material region
    GNaK : float, optional
        Value of GNaK, defaults to 1.0
    """
    SHORTNAME = 'luorudy94'
    MODELID = 'LuoRudy94'
    PARAMETERS = {'GNaK': 1.0}


class MacCannellIonicModel(_AbstractIonicModel):
    """
    Describes the MacCannell ionic model

    Parameters
    ----------
    IDs : list of int
        The material tags to be included in this IMP region
    name : str
        The name of the model material region
    GbNa : float, optional
        Value of GbNa, defaults to 0.0095
    GK1 : float, optional
        Value of GK1, defaults to 0.4822
    GKv : float, optional
        Value of GKv, defaults to 0.25
    Ki_init : float, optional
        Value of Ki_init, defaults to 140.045865
    Ko : float, optional
        Value of Ko, defaults to 5.4
    maxINaK : float, optional
        Value of maxINaK, defaults to 1.4
    Nai_init : float, optional
        Value of Nai_init, defaults to 9.954213
    Nao : float, optional
        Value of Nao, defaults to 130.0
    """
    SHORTNAME = 'maccannell'
    MODELID = 'MacCannell'
    PARAMETERS = {'GbNa': 0.0095,  'GK1': 0.4822,  'GKv': 0.25,  
                  'Ki_init': 140.045865,  'Ko': 5.4,  'maxINaK': 1.4,  
                  'Nai_init': 9.954213,  'Nao': 130.0}


class MahajanShiferawIonicModel(_AbstractIonicModel):
    """
    Describes the MahajanShiferaw ionic model

    Parameters
    ----------
    IDs : list of int
        The material tags to be included in this IMP region
    name : str
        The name of the model material region
    BASE_Ki : float, optional
        Value of BASE_Ki, defaults to 140.0
    Cao : float, optional
        Value of Cao
    constant_Ki : float, optional
        Value of constant_Ki, defaults to 0.0
    factorGKs : float, optional
        Value of factorGKs
    GCaL : float, optional
        Value of GCaL, defaults to 182.0
    GK1 : float, optional
        Value of GK1, defaults to 0.3
    GKr : float, optional
        Value of GKr, defaults to 0.0125
    GKs : float, optional
        Value of GKs, defaults to 0.32
    GNa : float, optional
        Value of GNa, defaults to 12.0
    GNaCa : float, optional
        Value of GNaCa, defaults to 0.84
    gss : float, optional
        Value of gss
    Gtof : float, optional
        Value of Gtof, defaults to 0.11
    Gtos : float, optional
        Value of Gtos, defaults to 0.04
    Ko : float, optional
        Value of Ko
    Nai : float, optional
        Value of Nai
    Nao : float, optional
        Value of Nao
    T : float, optional
        Value of T, defaults to 308.0
    """
    SHORTNAME = 'mahajanshiferaw'
    MODELID = 'MahajanShiferaw'
    PARAMETERS = {'BASE_Ki': 140.0,  'Cao': None,  'constant_Ki': 0.0,  
                  'factorGKs': None,  'GCaL': 182.0,  'GK1': 0.3,  
                  'GKr': 0.0125,  'GKs': 0.32,  'GNa': 12.0,  'GNaCa': 0.84,  
                  'gss': None,  'Gtof': 0.11,  'Gtos': 0.04,  'Ko': None,  
                  'Nai': None,  'Nao': None,  'T': 308.0}


class MaleckarIonicModel(_AbstractIonicModel):
    """
    Describes the Maleckar ionic model

    Parameters
    ----------
    IDs : list of int
        The material tags to be included in this IMP region
    name : str
        The name of the model material region
    type : float, optional
        Value of type, defaults to 1.0
    """
    SHORTNAME = 'maleckar'
    MODELID = 'Maleckar'
    PARAMETERS = {'type': 1.0}


class MitchellSchaefferIonicModel(_AbstractIonicModel):
    """
    Describes the MitchellSchaeffer ionic model

    Parameters
    ----------
    IDs : list of int
        The material tags to be included in this IMP region
    name : str
        The name of the model material region
    a_crit : float, optional
        Value of a_crit, defaults to 0.0
    tau_close : float, optional
        Value of tau_close, defaults to 150.0
    tau_in : float, optional
        Value of tau_in, defaults to 0.3
    tau_open : float, optional
        Value of tau_open, defaults to 120.0
    tau_out : float, optional
        Value of tau_out, defaults to 5.0
    V_gate : float, optional
        Value of V_gate, defaults to 0.13
    V_max : float, optional
        Value of V_max, defaults to 1.0
    V_min : float, optional
        Value of V_min, defaults to 0.0
    """
    SHORTNAME = 'mitchellschaeffer'
    MODELID = 'MitchellSchaeffer'
    PARAMETERS = {'a_crit': 0.0,  'tau_close': 150.0,  'tau_in': 0.3,  
                  'tau_open': 120.0,  'tau_out': 5.0,  'V_gate': 0.13,  
                  'V_max': 1.0,  'V_min': 0.0}


class NygrenIonicModel(_AbstractIonicModel):
    """
    Describes the Nygren ionic model

    Parameters
    ----------
    IDs : list of int
        The material tags to be included in this IMP region
    name : str
        The name of the model material region
    GCaB : float, optional
        Value of GCaB, defaults to 0.00157362
    GCaL : float, optional
        Value of GCaL, defaults to 0.135
    GK1 : float, optional
        Value of GK1, defaults to 0.06
    GKr : float, optional
        Value of GKr, defaults to 0.01
    GKS : float, optional
        Value of GKS, defaults to 0.02
    GNaB : float, optional
        Value of GNaB, defaults to 0.00121198
    Gsus : float, optional
        Value of Gsus, defaults to 0.055
    Gt : float, optional
        Value of Gt, defaults to 0.15
    KNaCa : float, optional
        Value of KNaCa, defaults to 0.000749684
    maxINaK : float, optional
        Value of maxINaK, defaults to 1.416506
    """
    SHORTNAME = 'nygren'
    MODELID = 'Nygren'
    PARAMETERS = {'GCaB': 0.00157362,  'GCaL': 0.135,  'GK1': 0.06,  
                  'GKr': 0.01,  'GKS': 0.02,  'GNaB': 0.00121198,  
                  'Gsus': 0.055,  'Gt': 0.15,  'KNaCa': 0.000749684,  
                  'maxINaK': 1.416506}


class OHaraIonicModel(_AbstractIonicModel):
    """
    Describes the OHara ionic model

    Parameters
    ----------
    IDs : list of int
        The material tags to be included in this IMP region
    name : str
        The name of the model material region
    Cao : float, optional
        Value of Cao, defaults to 1.8
    celltype : float, optional
        Value of celltype
    factorIbCa : float, optional
        Value of factorIbCa, defaults to 1.0
    factorIbNa : float, optional
        Value of factorIbNa, defaults to 1.0
    factorICaK : float, optional
        Value of factorICaK, defaults to 1.0
    factorICaL : float, optional
        Value of factorICaL, defaults to 1.0
    factorICaNa : float, optional
        Value of factorICaNa, defaults to 1.0
    factorINaCa : float, optional
        Value of factorINaCa, defaults to 1.0
    factorINaCass : float, optional
        Value of factorINaCass, defaults to 1.0
    factorINaK : float, optional
        Value of factorINaK, defaults to 1.0
    flags : float, optional
        Value of flags
    GK1 : float, optional
        Value of GK1
    GKb : float, optional
        Value of GKb
    GKr : float, optional
        Value of GKr
    GKs : float, optional
        Value of GKs
    GNa : float, optional
        Value of GNa
    GNaCa : float, optional
        Value of GNaCa
    GNaL : float, optional
        Value of GNaL
    GpCa : float, optional
        Value of GpCa, defaults to 0.0005
    Gto : float, optional
        Value of Gto
    INa_Type : float, optional
        Value of INa_Type
    jrel_stiff_const : float, optional
        Value of jrel_stiff_const, defaults to 0.005
    Ko : float, optional
        Value of Ko, defaults to 5.4
    modelformulation : float, optional
        Value of modelformulation
    Nao : float, optional
        Value of Nao, defaults to 140.0
    scale_tau_m : float, optional
        Value of scale_tau_m, defaults to 0.3
    shift_m_inf : float, optional
        Value of shift_m_inf, defaults to -8.0
    vHalfXs : float, optional
        Value of vHalfXs, defaults to 11.6
    """
    SHORTNAME = 'ohara'
    MODELID = 'OHara'
    PARAMETERS = {'Cao': 1.8,  'celltype': None,  'factorIbCa': 1.0,  
                  'factorIbNa': 1.0,  'factorICaK': 1.0,  'factorICaL': 1.0,  
                  'factorICaNa': 1.0,  'factorINaCa': 1.0,  
                  'factorINaCass': 1.0,  'factorINaK': 1.0,  'flags': None,  
                  'GK1': None,  'GKb': None,  'GKr': None,  'GKs': None,  
                  'GNa': None,  'GNaCa': None,  'GNaL': None,  'GpCa': 0.0005,  
                  'Gto': None,  'INa_Type': None,  'jrel_stiff_const': 0.005,  
                  'Ko': 5.4,  'modelformulation': None,  'Nao': 140.0,  
                  'scale_tau_m': 0.3,  'shift_m_inf': -8.0,  'vHalfXs': 11.6}


class PathmanathanIonicModel(_AbstractIonicModel):
    """
    Describes the Pathmanathan ionic model

    Parameters
    ----------
    IDs : list of int
        The material tags to be included in this IMP region
    name : str
        The name of the model material region
    beta : float, optional
        Value of beta, defaults to 1.0
    Cm : float, optional
        Value of Cm, defaults to 200.0
    xi : float, optional
        Value of xi, defaults to 3.0
    """
    SHORTNAME = 'pathmanathan'
    MODELID = 'Pathmanathan'
    PARAMETERS = {'beta': 1.0,  'Cm': 200.0,  'xi': 3.0}


class PlonseyIonicModel(_AbstractIonicModel):
    """
    Describes the Plonsey ionic model

    Parameters
    ----------
    IDs : list of int
        The material tags to be included in this IMP region
    name : str
        The name of the model material region
    Rm : float, optional
        Value of Rm, defaults to 10.0
    Vrest : float, optional
        Value of Vrest, defaults to 0.0
    """
    SHORTNAME = 'plonsey'
    MODELID = 'Plonsey'
    PARAMETERS = {'Rm': 10.0,  'Vrest': 0.0}


class RamirezIonicModel(_AbstractIonicModel):
    """
    Describes the Ramirez ionic model

    Parameters
    ----------
    IDs : list of int
        The material tags to be included in this IMP region
    name : str
        The name of the model material region
    Cao : float, optional
        Value of Cao, defaults to 1.8
    Clo : float, optional
        Value of Clo, defaults to 132.0
    GbCa : float, optional
        Value of GbCa, defaults to 0.00113
    GbNa : float, optional
        Value of GbNa, defaults to 0.000674
    GCaL : float, optional
        Value of GCaL, defaults to 0.24
    GClCa : float, optional
        Value of GClCa, defaults to 0.3
    GK1 : float, optional
        Value of GK1, defaults to 0.15
    GKr : float, optional
        Value of GKr, defaults to 0.06984
    GKs : float, optional
        Value of GKs, defaults to 0.0561
    GNa : float, optional
        Value of GNa, defaults to 7.8
    Gto : float, optional
        Value of Gto, defaults to 0.19824
    Ko : float, optional
        Value of Ko, defaults to 5.4
    maxCa_up : float, optional
        Value of maxCa_up, defaults to 15.0
    maxINaCa : float, optional
        Value of maxINaCa, defaults to 1600.0
    maxINaK : float, optional
        Value of maxINaK, defaults to 0.6
    maxIpCa : float, optional
        Value of maxIpCa, defaults to 0.275
    maxIup : float, optional
        Value of maxIup, defaults to 0.005
    Nao : float, optional
        Value of Nao, defaults to 140.0
    tau_tr : float, optional
        Value of tau_tr, defaults to 180.0
    tau_u : float, optional
        Value of tau_u, defaults to 8.0
    """
    SHORTNAME = 'ramirez'
    MODELID = 'Ramirez'
    PARAMETERS = {'Cao': 1.8,  'Clo': 132.0,  'GbCa': 0.00113,  
                  'GbNa': 0.000674,  'GCaL': 0.24,  'GClCa': 0.3,  'GK1': 0.15,  
                  'GKr': 0.06984,  'GKs': 0.0561,  'GNa': 7.8,  'Gto': 0.19824,  
                  'Ko': 5.4,  'maxCa_up': 15.0,  'maxINaCa': 1600.0,  
                  'maxINaK': 0.6,  'maxIpCa': 0.275,  'maxIup': 0.005,  
                  'Nao': 140.0,  'tau_tr': 180.0,  'tau_u': 8.0}


class SeveriIonicModel(_AbstractIonicModel):
    """
    Describes the Severi ionic model

    Parameters
    ----------
    IDs : list of int
        The material tags to be included in this IMP region
    name : str
        The name of the model material region
    ACh : float, optional
        Value of ACh, defaults to 0.0
    Cao : float, optional
        Value of Cao, defaults to 1.8
    CCh_cas : float, optional
        Value of CCh_cas, defaults to 0.0
    Cs_5_mM : float, optional
        Value of Cs_5_mM, defaults to 0.0
    Gf_K_max : float, optional
        Value of Gf_K_max
    Gf_Na_max : float, optional
        Value of Gf_Na_max
    GKr_max : float, optional
        Value of GKr_max, defaults to 0.0021637
    GKs_max : float, optional
        Value of GKs_max
    GNa_max : float, optional
        Value of GNa_max, defaults to 0.0125
    Gto_max : float, optional
        Value of Gto_max, defaults to 0.002
    INaK_max : float, optional
        Value of INaK_max, defaults to 0.063
    Iso_1_uM : float, optional
        Value of Iso_1_uM, defaults to 0.0
    Iso_cas : float, optional
        Value of Iso_cas, defaults to -1.0
    Iva_3_uM : float, optional
        Value of Iva_3_uM, defaults to 0.0
    K_Iso_inc_cas_ICaL : float, optional
        Value of K_Iso_inc_cas_ICaL, defaults to 1.0
    K_Iso_inc_cas_IKs : float, optional
        Value of K_Iso_inc_cas_IKs, defaults to 1.0
    K_Iso_inc_cas_INaK : float, optional
        Value of K_Iso_inc_cas_INaK, defaults to 1.0
    K_Iso_shift_cas_ICaL_dL_gate : float, optional
        Value of K_Iso_shift_cas_ICaL_dL_gate, defaults to 1.0
    K_Iso_shift_cas_If_y_gate : float, optional
        Value of K_Iso_shift_cas_If_y_gate, defaults to 1.0
    K_Iso_shift_cas_IKs_n_gate : float, optional
        Value of K_Iso_shift_cas_IKs_n_gate, defaults to 1.0
    K_Iso_slope_cas_ICaL : float, optional
        Value of K_Iso_slope_cas_ICaL, defaults to 1.0
    K_j_SRCarel : float, optional
        Value of K_j_SRCarel, defaults to 1.0
    K_j_up : float, optional
        Value of K_j_up, defaults to 1.0
    K_k1 : float, optional
        Value of K_k1, defaults to 0.92
    K_koCa : float, optional
        Value of K_koCa, defaults to 1.0
    K_NaCa : float, optional
        Value of K_NaCa, defaults to 4.0
    P_CaL : float, optional
        Value of P_CaL, defaults to 0.2
    P_CaT : float, optional
        Value of P_CaT, defaults to 0.02
    """
    SHORTNAME = 'severi'
    MODELID = 'Severi'
    PARAMETERS = {'ACh': 0.0,  'Cao': 1.8,  'CCh_cas': 0.0,  'Cs_5_mM': 0.0,  
                  'Gf_K_max': None,  'Gf_Na_max': None,  'GKr_max': 0.0021637,  
                  'GKs_max': None,  'GNa_max': 0.0125,  'Gto_max': 0.002,  
                  'INaK_max': 0.063,  'Iso_1_uM': 0.0,  'Iso_cas': -1.0,  
                  'Iva_3_uM': 0.0,  'K_Iso_inc_cas_ICaL': 1.0,  
                  'K_Iso_inc_cas_IKs': 1.0,  'K_Iso_inc_cas_INaK': 1.0,  
                  'K_Iso_shift_cas_ICaL_dL_gate': 1.0,  
                  'K_Iso_shift_cas_If_y_gate': 1.0,  
                  'K_Iso_shift_cas_IKs_n_gate': 1.0,  
                  'K_Iso_slope_cas_ICaL': 1.0,  'K_j_SRCarel': 1.0,  
                  'K_j_up': 1.0,  'K_k1': 0.92,  'K_koCa': 1.0,  'K_NaCa': 4.0,  
                  'P_CaL': 0.2,  'P_CaT': 0.02}


class ShannonIonicModel(_AbstractIonicModel):
    """
    Describes the Shannon ionic model

    Parameters
    ----------
    IDs : list of int
        The material tags to be included in this IMP region
    name : str
        The name of the model material region
    """
    SHORTNAME = 'shannon'
    MODELID = 'Shannon'
    PARAMETERS = {}


class SkibsbyeIonicModel(_AbstractIonicModel):
    """
    Describes the Skibsbye ionic model

    Parameters
    ----------
    IDs : list of int
        The material tags to be included in this IMP region
    name : str
        The name of the model material region
    ACh : float, optional
        Value of ACh, defaults to 0.0
    cAF_cpumps : float, optional
        Value of cAF_cpumps, defaults to 1.0
    cAF_gCaL : float, optional
        Value of cAF_gCaL, defaults to 1.0
    cAF_gK1 : float, optional
        Value of cAF_gK1, defaults to 1.0
    cAF_gKCa : float, optional
        Value of cAF_gKCa, defaults to 1.0
    cAF_gKs : float, optional
        Value of cAF_gKs, defaults to 1.0
    cAF_gNa : float, optional
        Value of cAF_gNa, defaults to 1.0
    cAF_gsus : float, optional
        Value of cAF_gsus, defaults to 1.0
    cAF_gt : float, optional
        Value of cAF_gt, defaults to 1.0
    cAF_kNaCa : float, optional
        Value of cAF_kNaCa, defaults to 1.0
    cAF_lcell : float, optional
        Value of cAF_lcell, defaults to 1.0
    cAF_phos : float, optional
        Value of cAF_phos, defaults to 1.0
    cAF_PLB : float, optional
        Value of cAF_PLB, defaults to 1.0
    cAF_RyR : float, optional
        Value of cAF_RyR, defaults to 1.0
    cAF_SLN : float, optional
        Value of cAF_SLN, defaults to 1.0
    factorgcal : float, optional
        Value of factorgcal, defaults to 1.0
    factorgk1 : float, optional
        Value of factorgk1, defaults to 1.0
    factorgkr : float, optional
        Value of factorgkr, defaults to 1.0
    factorgks : float, optional
        Value of factorgks, defaults to 1.0
    factorgto : float, optional
        Value of factorgto, defaults to 1.0
    """
    SHORTNAME = 'skibsbye'
    MODELID = 'Skibsbye'
    PARAMETERS = {'ACh': 0.0,  'cAF_cpumps': 1.0,  'cAF_gCaL': 1.0,  
                  'cAF_gK1': 1.0,  'cAF_gKCa': 1.0,  'cAF_gKs': 1.0,  
                  'cAF_gNa': 1.0,  'cAF_gsus': 1.0,  'cAF_gt': 1.0,  
                  'cAF_kNaCa': 1.0,  'cAF_lcell': 1.0,  'cAF_phos': 1.0,  
                  'cAF_PLB': 1.0,  'cAF_RyR': 1.0,  'cAF_SLN': 1.0,  
                  'factorgcal': 1.0,  'factorgk1': 1.0,  'factorgkr': 1.0,  
                  'factorgks': 1.0,  'factorgto': 1.0}


class StewartIonicModel(_AbstractIonicModel):
    """
    Describes the Stewart ionic model

    Parameters
    ----------
    IDs : list of int
        The material tags to be included in this IMP region
    name : str
        The name of the model material region
    Gbca : float, optional
        Value of Gbca, defaults to 0.000592
    Gbna : float, optional
        Value of Gbna, defaults to 0.00029
    GCaL : float, optional
        Value of GCaL, defaults to 3.98e-05
    Gf_K : float, optional
        Value of Gf_K, defaults to 0.0234346
    Gf_Na : float, optional
        Value of Gf_Na, defaults to 0.0145654
    GK1 : float, optional
        Value of GK1, defaults to 0.065
    GKr : float, optional
        Value of GKr, defaults to 0.0918
    GKs : float, optional
        Value of GKs, defaults to 0.2352
    GNa : float, optional
        Value of GNa, defaults to 130.5744
    GpCa : float, optional
        Value of GpCa, defaults to 0.1238
    GpK : float, optional
        Value of GpK, defaults to 0.0146
    Gsus : float, optional
        Value of Gsus, defaults to 0.0227
    Gto : float, optional
        Value of Gto, defaults to 0.08184
    maxINaCa : float, optional
        Value of maxINaCa, defaults to 1000.0
    maxINaK : float, optional
        Value of maxINaK, defaults to 2.724
    """
    SHORTNAME = 'stewart'
    MODELID = 'Stewart'
    PARAMETERS = {'Gbca': 0.000592,  'Gbna': 0.00029,  'GCaL': 3.98e-05,  
                  'Gf_K': 0.0234346,  'Gf_Na': 0.0145654,  'GK1': 0.065,  
                  'GKr': 0.0918,  'GKs': 0.2352,  'GNa': 130.5744,  
                  'GpCa': 0.1238,  'GpK': 0.0146,  'Gsus': 0.0227,  
                  'Gto': 0.08184,  'maxINaCa': 1000.0,  'maxINaK': 2.724}


class TomekIonicModel(_AbstractIonicModel):
    """
    Describes the Tomek ionic model

    Parameters
    ----------
    IDs : list of int
        The material tags to be included in this IMP region
    name : str
        The name of the model material region
    celltype : float, optional
        Value of celltype
    flags : float, optional
        Value of flags
    """
    SHORTNAME = 'tomek'
    MODELID = 'Tomek'
    PARAMETERS = {'celltype': None,  'flags': None}


class WangSobieIonicModel(_AbstractIonicModel):
    """
    Describes the WangSobie ionic model

    Parameters
    ----------
    IDs : list of int
        The material tags to be included in this IMP region
    name : str
        The name of the model material region
    Cai_init : float, optional
        Value of Cai_init, defaults to 0.2049
    ICaL_slowdown : float, optional
        Value of ICaL_slowdown, defaults to 1.0
    Ki_init : float, optional
        Value of Ki_init, defaults to 136450.0
    Nai_init : float, optional
        Value of Nai_init, defaults to 21747.0
    """
    SHORTNAME = 'wangsobie'
    MODELID = 'WangSobie'
    PARAMETERS = {'Cai_init': 0.2049,  'ICaL_slowdown': 1.0,  
                  'Ki_init': 136450.0,  'Nai_init': 21747.0}


class tenTusscherPanfilovIonicModel(_AbstractIonicModel):
    """
    Describes the tenTusscherPanfilov ionic model

    Parameters
    ----------
    IDs : list of int
        The material tags to be included in this IMP region
    name : str
        The name of the model material region
    Bufc : float, optional
        Value of Bufc, defaults to 0.2
    Bufsr : float, optional
        Value of Bufsr, defaults to 10.0
    Bufss : float, optional
        Value of Bufss, defaults to 0.4
    Cao : float, optional
        Value of Cao, defaults to 2.0
    CAPACITANCE : float, optional
        Value of CAPACITANCE, defaults to 0.185
    cell_type : float, optional
        Value of cell_type
    D_CaL_off : float, optional
        Value of D_CaL_off, defaults to 0.0
    Fconst : float, optional
        Value of Fconst, defaults to 96485.3415
    flags : float, optional
        Value of flags
    GbCa : float, optional
        Value of GbCa, defaults to 0.000592
    GbNa : float, optional
        Value of GbNa, defaults to 0.00029
    GCaL : float, optional
        Value of GCaL, defaults to 3.98e-05
    GK1 : float, optional
        Value of GK1, defaults to 5.405
    GKr : float, optional
        Value of GKr, defaults to 0.153
    GKs : float, optional
        Value of GKs
    GNa : float, optional
        Value of GNa, defaults to 14.838
    GpCa : float, optional
        Value of GpCa, defaults to 0.1238
    GpK : float, optional
        Value of GpK, defaults to 0.0146
    Gto : float, optional
        Value of Gto
    Kbufc : float, optional
        Value of Kbufc, defaults to 0.001
    Kbufsr : float, optional
        Value of Kbufsr, defaults to 0.3
    Kbufss : float, optional
        Value of Kbufss, defaults to 0.00025
    KmCa : float, optional
        Value of KmCa, defaults to 1.38
    KmK : float, optional
        Value of KmK, defaults to 1.0
    KmNa : float, optional
        Value of KmNa, defaults to 40.0
    KmNai : float, optional
        Value of KmNai, defaults to 87.5
    knaca : float, optional
        Value of knaca, defaults to 1000.0
    knak : float, optional
        Value of knak, defaults to 2.724
    Ko : float, optional
        Value of Ko, defaults to 5.4
    KpCa : float, optional
        Value of KpCa, defaults to 0.0005
    ksat : float, optional
        Value of ksat, defaults to 0.1
    Kup : float, optional
        Value of Kup, defaults to 0.00025
    n : float, optional
        Value of n, defaults to 0.35
    Nao : float, optional
        Value of Nao, defaults to 140.0
    pKNa : float, optional
        Value of pKNa, defaults to 0.03
    Rconst : float, optional
        Value of Rconst, defaults to 8314.472
    scl_tau_f : float, optional
        Value of scl_tau_f, defaults to 1.0
    T : float, optional
        Value of T, defaults to 310.0
    Vc : float, optional
        Value of Vc, defaults to 0.016404
    vHalfXs : float, optional
        Value of vHalfXs, defaults to 5.0
    Vleak : float, optional
        Value of Vleak, defaults to 0.00036
    Vmaxup : float, optional
        Value of Vmaxup, defaults to 0.006375
    Vrel : float, optional
        Value of Vrel, defaults to 0.102
    Vsr : float, optional
        Value of Vsr, defaults to 0.001094
    Vss : float, optional
        Value of Vss, defaults to 5.468e-05
    xr2_off : float, optional
        Value of xr2_off, defaults to 0.0
    """
    SHORTNAME = 'tentusscherpanfilov'
    MODELID = 'tenTusscherPanfilov'
    PARAMETERS = {'Bufc': 0.2,  'Bufsr': 10.0,  'Bufss': 0.4,  'Cao': 2.0,  
                  'CAPACITANCE': 0.185,  'cell_type': None,  'D_CaL_off': 0.0,  
                  'Fconst': 96485.3415,  'flags': None,  'GbCa': 0.000592,  
                  'GbNa': 0.00029,  'GCaL': 3.98e-05,  'GK1': 5.405,  
                  'GKr': 0.153,  'GKs': None,  'GNa': 14.838,  'GpCa': 0.1238,  
                  'GpK': 0.0146,  'Gto': None,  'Kbufc': 0.001,  'Kbufsr': 0.3,  
                  'Kbufss': 0.00025,  'KmCa': 1.38,  'KmK': 1.0,  'KmNa': 40.0,  
                  'KmNai': 87.5,  'knaca': 1000.0,  'knak': 2.724,  'Ko': 5.4,  
                  'KpCa': 0.0005,  'ksat': 0.1,  'Kup': 0.00025,  'n': 0.35,  
                  'Nao': 140.0,  'pKNa': 0.03,  'Rconst': 8314.472,  
                  'scl_tau_f': 1.0,  'T': 310.0,  'Vc': 0.016404,  
                  'vHalfXs': 5.0,  'Vleak': 0.00036,  'Vmaxup': 0.006375,  
                  'Vrel': 0.102,  'Vsr': 0.001094,  'Vss': 5.468e-05,  
                  'xr2_off': 0.0}


MAPPING = _OrderedDict()
MAPPING['passive']             = PassiveIonicRegion
MAPPING['alievpanfilov']       = AlievPanfilovIonicModel
MAPPING['aslanidisleiman']     = AslanidiSleimanIonicModel
MAPPING['augustin']            = AugustinIonicModel
MAPPING['bondarenko']          = BondarenkoIonicModel
MAPPING['campos']              = CamposIonicModel
MAPPING['courtemanche']        = CourtemancheIonicModel
MAPPING['difrancesconoble']    = DiFrancescoNobleIonicModel
MAPPING['drouhardroberge']     = DrouhardRobergeIonicModel
MAPPING['fabbri']              = FabbriIonicModel
MAPPING['fox']                 = FoxIonicModel
MAPPING['gaur']                = GaurIonicModel
MAPPING['grandi']              = GrandiIonicModel
MAPPING['grandipanditvoigt']   = GrandiPanditVoigtIonicModel
MAPPING['hodgkinhuxley']       = HodgkinHuxleyIonicModel
MAPPING['inada']               = InadaIonicModel
MAPPING['iribekohl']           = IribeKohlIonicModel
MAPPING['kurata']              = KurataIonicModel
MAPPING['loewe']               = LoeweIonicModel
MAPPING['luorudy91']           = LuoRudy91IonicModel
MAPPING['luorudy94']           = LuoRudy94IonicModel
MAPPING['maccannell']          = MacCannellIonicModel
MAPPING['mahajanshiferaw']     = MahajanShiferawIonicModel
MAPPING['maleckar']            = MaleckarIonicModel
MAPPING['mitchellschaeffer']   = MitchellSchaefferIonicModel
MAPPING['nygren']              = NygrenIonicModel
MAPPING['ohara']               = OHaraIonicModel
MAPPING['pathmanathan']        = PathmanathanIonicModel
MAPPING['plonsey']             = PlonseyIonicModel
MAPPING['ramirez']             = RamirezIonicModel
MAPPING['severi']              = SeveriIonicModel
MAPPING['shannon']             = ShannonIonicModel
MAPPING['skibsbye']            = SkibsbyeIonicModel
MAPPING['stewart']             = StewartIonicModel
MAPPING['tomek']               = TomekIonicModel
MAPPING['wangsobie']           = WangSobieIonicModel
MAPPING['tentusscherpanfilov'] = tenTusscherPanfilovIonicModel


def keys():
    """
    Return a list of string keys to be used with :func:`get`
    """
    return MAPPING.keys()


def get(*args, **kwargs):
    """
    Get a class from its key string from :func:`keys`
    """
    return MAPPING.get(*args, **kwargs)


def summary(indent=2, ncolumns=4, padding=2):
    """
    Return a string summarizing the models and their parameters
    """
    modelattr = ['SHORTNAME', 'MODELID']
    maxattrlen = max(map(len, modelattr+['PARAMETERS']))
    summarystr = 'IONIC MODELS\n\n'
    indentstr = ' '*indent
    for name, model in MAPPING.items():
        summarystr += indentstr+'{}\n'.format(name)
        for attr in modelattr:
            obj = getattr(model, attr, None)
            if obj is None:
                continue
            summarystr += indentstr*2+'{}: {}\n'.format(attr.lower().ljust(maxattrlen), str(obj))
        attr = 'PARAMETERS'
        obj = getattr(model, attr, None)
        if obj is not None and len(obj) > 0:
            tablestr = stringtable(obj, ncolumns, padding, rowprefix=' '*(maxattrlen+2*indent+2))
            summarystr += indentstr*2+attr.lower().ljust(maxattrlen)+': '+tablestr[len(attr)+2*indent+2:]
        summarystr += '\n'
    return summarystr
